@@ -1,13 +1,16 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
+"""
+Main script for the patchbot client.
 
-####################################################################
-#
-# This is the main script for the patchbot. It pulls branches from
-# trac, applies them, and publishes the results of the tests to a
-# server running serve.py.  Configuration is primarily done via an
-# optional conf.txt file passed in as a command line argument.
-#
+This is the main script for the patchbot client. It pulls branches
+from trac, applies them, and publishes the results of the tests to a
+server running ``serve.py``.  Configuration is primarily done via an
+optional ``conf.json`` file (json format) passed in as a command line
+argument.
+"""
+
+# -------------------------------------------------------------------
 #          Author: Robert Bradshaw <robertwb@gmail.com>
 #
 #               Copyright 2010-14 (C) Google, Inc.
@@ -16,8 +19,10 @@
 #  as published by the Free Software Foundation; either version 2 of
 #  the License, or (at your option) any later version.
 #                  http://www.gnu.org/licenses/
-####################################################################
+# -------------------------------------------------------------------
 
+
+# global python imports
 import codecs
 import hashlib
 import signal
@@ -52,19 +57,18 @@ except ImportError:
     from urllib.parse import urlencode
 
 from datetime import datetime
-
 from optparse import OptionParser
-from http_post_file import post_multipart
-from trac import scrape, pull_from_trac
-from trac import TracServer, Config
+
+# imports from patchbot sources
+from trac import scrape, pull_from_trac, TracServer, Config
 from util import (now_str, prune_pending, do_or_die,
-                  get_version, current_reports, git_commit,
+                  get_sage_version, current_reports, git_commit,
                   describe_branch, compare_version, temp_build_suffix,
                   ensure_free_space,
                   ConfigException, SkipTicket)
-import version as patchbot_version
+from http_post_file import post_multipart
 from plugins import PluginResult
-
+from version import get_patchbot_version
 
 # name of the log files
 LOG_RATING = 'rating.log'
@@ -103,9 +107,11 @@ def compare_machines(a, b, machine_match=None):
 
     machine_match is a number of initial things to look at.
 
-    >>> m1 = ['Ubuntu', '14.04', 'i686', '3.13.0-40-generic', 'arando']
-    >>> m2 = ['Fedora', '19', 'x86_64', '3.10.4-300.fc19.x86_64', 'desktop']
-    >>> compare_machines(m1, m2)
+    EXAMPLES::
+
+        >>> m1 = ['Ubuntu', '14.04', 'i686', '3.13.0-40-generic', 'arando']
+        >>> m2 = ['Fedora', '19', 'x86_64', '3.10.4-300.fc19.x86_64', 'desktop']
+        >>> compare_machines(m1, m2)
     """
     if machine_match is not None:
         a = a[:machine_match]
@@ -124,7 +130,7 @@ def alarm_handler(signum, frame):
     raise TimeOut
 
 
-class Tee:
+class Tee(object):
     def __init__(self, filepath, time=False, timeout=None, timer=None):
         if timeout is None:
             timeout = 60 * 60 * 24
@@ -170,7 +176,7 @@ class Tee:
         return False
 
 
-class Timer:
+class Timer(object):
     def __init__(self):
         self._starts = {}
         self._history = []
@@ -208,23 +214,23 @@ status = {'started': 'ApplyFailed',
           'skipped': 'Pending'}
 
 
-def boundary(name, type):
+def boundary(name, text_type):
     """
     Return text that bound parts of the reports.
 
-    Type can be 'plugin', 'plugin_end', 'ticket' and 'spkg'
+    type can be 'plugin', 'plugin_end', 'ticket' and 'spkg'
     """
-    if type == 'plugin':
+    if text_type == 'plugin':
         letter = '='
         length = 10
-    elif type == 'plugin_end':
+    elif text_type == 'plugin_end':
         name = 'end ' + name
         letter = '='
         length = 10
-    elif type == 'ticket':
+    elif text_type == 'ticket':
         letter = '='
         length = 30
-    elif type == 'spkg':
+    elif text_type == 'spkg':
         letter = '+'
         length = 10
     return ' '.join((letter * length, str(name), letter * length))
@@ -236,8 +242,10 @@ def machine_data():
 
     This uses ``uname`` to find the data.
 
-    m1 = ['Ubuntu', '14.04', 'i686', '3.13.0-40-generic', 'arando']
-    m2 = ['Fedora', '19', 'x86_64', '3.10.4-300.fc19.x86_64', 'desktop']
+    EXAMPLES::
+
+        m1 = ['Ubuntu', '14.04', 'i686', '3.13.0-40-generic', 'arando']
+        m2 = ['Fedora', '19', 'x86_64', '3.10.4-300.fc19.x86_64', 'desktop']
     """
     system, node, release, version, arch = os.uname()
     if system.lower() == "linux":
@@ -308,7 +316,7 @@ def sha1file(path, blocksize=None):
     return h.hexdigest()
 
 
-class Patchbot:
+class Patchbot(object):
     """
     Main class of the patchbot.
 
@@ -319,14 +327,12 @@ class Patchbot:
     - sage_root -- path to the sage local repository
     - server -- http address of the patchbot server
     - config_path -- ``None`` or path to the config file
-    - dry_run -- boolean
-    - plugin_only -- boolean
-    - options
+    - options -- an option class or ``None``
 
     EXAMPLES::
 
         >>> from patchbot import Patchbot
-        >>> P = Patchbot('/homes/leila/sage','http://patchbot.sagemath.org',None,False,True,None)
+        >>> P = Patchbot('/homes/leila/sage','http://patchbot.sagemath.org')
         >>> import os
         >>> os.chdir(P.sage_root)
         >>> P.test_a_ticket(12345)
@@ -337,18 +343,36 @@ class Patchbot:
 
     written inside the config.json file passed using --config=config.json
     """
-    def __init__(self, sage_root, server, config_path, dry_run,
-                 plugin_only, options):
+    def __init__(self, sage_root, server, config_path=None, options=None):
 
         self.sage_root = sage_root
         self.sage_command = os.path.join(self.sage_root, 'sage')
         self.server = server
         self.trac_server = TracServer(Config())
-        self.base = get_version(sage_root)
-        self.dry_run = dry_run
-        self.plugin_only = plugin_only
+        self.base = get_sage_version(sage_root)
         self.config_path = config_path
 
+        # preparing the options
+        if options is None:
+            # this default is for test of patchbot in ipython session
+            class opt(object):
+                # two attribute options
+                dry_run = False
+                plugin_only = True
+                # four other options
+                no_banner = False
+                owner = None
+                safe_only = True
+                skip_base = True
+            self.options = opt
+        else:
+            self.options = options
+
+        # two special options are set as attributes
+        self.dry_run = self.options.dry_run
+        self.plugin_only = self.options.plugin_only
+
+        # preparing the logs
         self.log_dir = os.path.join(self.sage_root, 'logs', 'patchbot')
         if not os.path.exists(self.log_dir):
             os.makedirs(self.log_dir)
@@ -357,7 +381,7 @@ class Patchbot:
                              encoding='utf8')
         handle.close()
 
-        self._version = patchbot_version.get_version()
+        self._version = get_patchbot_version()
         self.last_pull = 0
         self.to_skip = {}
 
@@ -365,17 +389,9 @@ class Patchbot:
             self._version,
             self.sage_root), LOG_MAIN)
 
+        # other options are handled here
         self.reload_config()
 
-        if options is None:
-            # ugly workaround to simplify interactive use of Patchbot
-
-            class opt:
-                safe_only = True
-            self.options = opt
-        else:
-            self.options = options
-
     def write_log(self, msg, logfile=None, date=True):
         r"""
         Write ``msg`` in a logfile.
@@ -434,12 +450,26 @@ class Patchbot:
         Return the version of the patchbot.
 
         Something like '2.3.7'
+
+        EXAMPLES::
+
+            In [3]: P.version()
+            Out[3]: u'2.5.3'
         """
         return self._version
 
     def banner(self):
         """
         A banner for the patchbot
+
+        EXAMPLES::
+
+            In [7]: print(P.banner())
+            ┌─┬──────┐
+            │░│ ⊙  ʘ │        SageMath patchbot
+            │░│      │
+            │░│ ──── │        version 2.5.3
+            ╘═╧══════╛
         """
         s = u'┌─┬──────┐\n'
         s += u'│░│ ⊙  ʘ │        SageMath patchbot\n'
@@ -491,7 +521,7 @@ class Patchbot:
             self._default_trusted = set(trusted)
             return self._default_trusted
 
-    def lookup_ticket(self, id, verbose=False):
+    def lookup_ticket(self, t_id, verbose=False):
         """
         Retrieve information about one ticket from the patchbot server.
 
@@ -506,7 +536,7 @@ class Patchbot:
         http://patchbot.sagemath.org/ticket/?raw&query={"id":11529}&pretty
         """
         path = "ticket/?" + urlencode({'raw': True,
-                                       'query': json.dumps({'id': id})})
+                                       'query': json.dumps({'id': t_id})})
         res = self.load_json_from_server(path, retry=3)
         if res:
             if verbose:
@@ -516,19 +546,29 @@ class Patchbot:
             # trying using trac server instead
             if verbose:
                 print('lookup using scrape')
-            return scrape(id)
+            return scrape(t_id)
 
     def get_config(self):
         """
         Return the configuration.
 
         Either by default, or from a json config file.
+
+        This is done in the following order:
+
+        1) give all keys the default values as stored in the code here
+
+        2) override with values from json config file if given
+
+        3) override from the options class if given
         """
+        # loading the json config file
         if self.config_path is None:
             unicode_conf = {}
         else:
             unicode_conf = json.load(open(self.config_path))
-        # defaults
+
+        # hardcoded defaults
         conf = {"idle": 300,
                 "time_of_day": "0-0",  # midnight-midnight
                 "parallelism": 3,
@@ -537,6 +577,7 @@ class Patchbot:
                             "plugins.coverage",
                             "plugins.non_ascii",
                             "plugins.doctest_continuation",
+                            "plugins.next_method",
                             "plugins.raise_statements",
                             "plugins.input_output_block",
                             "plugins.reference_block",
@@ -555,11 +596,15 @@ class Patchbot:
                 "keep_open_branches": True,
                 "base_repo": "git://github.com/sagemath/sage.git",
                 "base_branch": "develop",
-                "max_behind_commits": 1,
+                "max_behind_commits": 0,
                 "max_behind_days": 1.0,
                 "use_ccache": True,
+                # four options that can also be changed using sage --xx
+                "no_banner": False,
+                "owner": "unknown owner",
                 "safe_only": True,
                 "skip_base": False}
+
         default_bonus = {"needs_review": 1000,
                          "positive_review": 500,
                          "blocker": 100,
@@ -569,22 +614,37 @@ class Patchbot:
                          "applies": 20,
                          "behind": 1}
 
+        # now override the default by the config file
         for key, value in unicode_conf.items():
             conf[str(key)] = value
 
+        # complete back the bonus from config file if needed
         for key, value in default_bonus.items():
             if key not in conf['bonus']:
                 conf['bonus'][key] = value
 
+        # using default trust procedure if no trust list given
         if "trusted_authors" not in conf:
             conf["trusted_authors"] = self.default_trusted_authors()
         if "extra_trusted_authors" in conf:
             conf["trusted_authors"].update(conf["extra_trusted_authors"])
 
-        # force building the doc, so that the tests can pass
+        # now override with some values from options
+        # coming from sage -patchbot --xx
+        if self.options.no_banner is not None:
+            conf['no_banner'] = self.options.no_banner
+        if self.options.owner is not None:
+            conf['owner'] = self.options.owner
+        if self.options.safe_only is not None:
+            conf['safe_only'] = self.options.safe_only
+        if self.options.skip_base is not None:
+            conf['skip_base'] = self.options.skip_base
+
+        # plugin setup
         if not self.plugin_only:
             active_plugins = conf['plugins']
             if not "plugins.docbuild" in active_plugins:
+                # force building the doc, so that the tests can pass
                 conf['plugins'] = active_plugins + ["plugins.docbuild"]
 
         def locate_plugin(name):
@@ -594,9 +654,11 @@ class Patchbot:
             plugin = getattr(__import__(module, fromlist=[name]), name)
             assert callable(plugin)
             return plugin
+
         conf["plugins"] = [(name, locate_plugin(name))
                            for name in conf["plugins"]]
 
+        # logs
         self.delete_log(LOG_CONFIG)
         self.write_log("Configuration for the patchbot\n{}\n".format(now_str()), LOG_CONFIG, False)
         self.write_log(pprint.pformat(conf), LOG_CONFIG, False)
@@ -657,7 +719,7 @@ class Patchbot:
         Return the human name of the base branch.
         """
         # TODO: Is this stable?
-        version = get_version(self.sage_root)
+        version = get_sage_version(self.sage_root)
         commit_count = subprocess.check_output(['git', 'rev-list', '--count',
                                                 '%s..patchbot/base' % version])
         return "{} + {} commits".format(version, commit_count.strip())
@@ -679,27 +741,31 @@ class Patchbot:
         query = "raw&status={}".format(status)
 
         self.write_log("Getting ticket list...", LOG_MAIN)
-        all = self.load_json_from_server("ticket/?" + query, retry=10)
+        all_tickets = self.load_json_from_server("ticket/?" + query, retry=10)
+
+        # rating for all tickets
+        self.delete_log(LOG_RATING)
+        all_tickets = [(self.rate_ticket(ti, verbose=(verbose == 2)), ti)
+                       for ti in all_tickets]
 
         # remove all tickets with None rating
-        self.delete_log(LOG_RATING)
-        all = list(filter(lambda x: x[0] is not None,
-                          ((self.rate_ticket(t, verbose=(verbose == 2)), t)
-                           for t in all)))
+        all_tickets = [ti for ti in all_tickets if ti[0] is not None]
 
         # sort tickets using their ratings
-        all.sort()
+        all_tickets.sort()
 
         self.delete_log(LOG_RATING_SHORT)
         if verbose >= 1:
             logfile = [LOG_RATING_SHORT, sys.stdout]
         else:
             logfile = [LOG_RATING_SHORT]
-        for rating, ticket in reversed(all):
-            self.write_log(u'#{:<6}{:30}{}'.format(ticket['id'], str(rating[:2]), ticket['title']),
+        for rating, ticket in reversed(all_tickets):
+            self.write_log(u'#{:<6}{:30}{}'.format(ticket['id'],
+                                                   str(rating[:2]),
+                                                   ticket['title']),
                            logfile, date=False)
 
-        return all[-1]
+        return all_tickets[-1]
 
     def rate_ticket(self, ticket, verbose=False):
         """
@@ -842,6 +908,11 @@ class Patchbot:
     def current_reports(self, ticket, newer=False):
         """
         Return the list of current reports on a ticket.
+
+        EXAMPLES::
+
+            In [4]: P.current_reports(20240)
+            Out[4]: []
         """
         if isinstance(ticket, (int, str)):
             ticket = self.lookup_ticket(ticket)
@@ -901,12 +972,13 @@ class Patchbot:
         if not self.plugin_only:
             self.report_ticket(ticket, status='Pending', log=log)
         plugins_results = []
-        print(self.banner().encode('utf8'))
+        if not self.config['no_banner']:
+            print(self.banner().encode('utf8'))
         botmake = os.getenv('MAKE', "make -j{}".format(self.config['parallelism']))
         os.environ['SAGE_ROOT'] = self.sage_root
         os.environ['GIT_AUTHOR_NAME'] = os.environ['GIT_COMMITTER_NAME'] = 'patchbot'
         os.environ['GIT_AUTHOR_EMAIL'] = os.environ['GIT_COMMITTER_EMAIL'] = 'patchbot@localhost'
-        os.environ['GIT_AUTHOR_DATE'] = os.environ['GIT_COMMITTER_DATE'] = '1970-01-01T00:00:00'
+        os.environ['GIT_AUTHOR_DATE'] = os.environ['GIT_COMMITTER_DATE'] = '1970-01-01T00:00:01'
         try:
             t = Timer()
             with Tee(log, time=True, timeout=self.config['timeout'], timer=t):
@@ -1022,11 +1094,11 @@ class Patchbot:
                             state = 'tests_passed_plugins_failed'
 
         except (HTTPError, socket.error, ConfigException):
-            # Don't report failure because the network/trac died...
+            # Do not report failure because the network/trac died...
             self.write_log('network failure... skip this ticket', LOG_MAIN)
             t.print_all()
             traceback.print_exc()
-            # Don't try this again for at least an hour.
+            # Do not try this again for at least an hour.
             self.to_skip[ticket['id']] = time.time() + 60 * 60
             state = 'network_error'
         except SkipTicket as exn:
@@ -1043,7 +1115,7 @@ class Patchbot:
             traceback.print_exc()
             self.to_skip[ticket['id']] = time.time() + 12 * 60 * 60
         except:
-            # Don't try this again for a while.
+            # Do not try this again for a while.
             self.to_skip[ticket['id']] = time.time() + 12 * 60 * 60
             raise
 
@@ -1204,10 +1276,13 @@ class Patchbot:
             report['base'] = ticket_base = sorted(tags, compare_version)[-1]
             report['git_base'] = self.git_commit('patchbot/base')
             report['git_base_human'] = describe_branch('patchbot/base')
+            if ticket['id'] == 0:
+                report['owner'] = self.owner
             if ticket['id'] != 0:
                 report['git_branch'] = ticket.get('git_branch', None)
                 report['git_log'] = subprocess.check_output(['git', 'log', '--oneline', '%s..patchbot/ticket_upstream' % ticket_base]).strip().split('\n')
-                # If apply failed, we don't want to be stuck in an infinite loop.
+                # If apply failed, we do not want to be stuck in an
+                # infinite loop.
                 report['git_commit'] = self.git_commit('patchbot/ticket_upstream')
                 report['git_commit_human'] = describe_branch('patchbot/ticket_upstream')
                 report['git_merge'] = self.git_commit('patchbot/ticket_merged')
@@ -1242,56 +1317,75 @@ class Patchbot:
 
 def main(args):
     """
-    Most configuration is done in the config file, which is reread between
-    each ticket for live configuration of the patchbot.
+    Most configuration is done in the json config file, which is
+    reread between each ticket for live configuration of the patchbot.
     """
     global conf
     parser = OptionParser()
-    parser.add_option("--config", dest="config",
-                      help="specify the config file")
+
+    # options that are passed as arguments to the class "Patchbot"
     parser.add_option("--sage-root", dest="sage_root",
                       default=os.environ.get('SAGE_ROOT'),
                       help="specify another sage root directory")
     parser.add_option("--server", dest="server",
                       default="http://patchbot.sagemath.org/",
                       help="specify another patchbot server adress")
-    parser.add_option("--count", dest="count", default=1000000)
-    parser.add_option("--ticket", dest="ticket", default=None,
-                      help="test only a list of tickets, for example '12345,19876'")
-    parser.add_option("--list", dest="list", action="store_true", default=False,
+    parser.add_option("--config", dest="config",
+                      help="specify the json config file")
+
+    # options that are only used directly here
+    parser.add_option("--cleanup", action="store_true", dest="cleanup",
+                      default=False,
+                      help="whether to cleanup the temporary files")
+    parser.add_option("--count", dest="count",
+                      default=1000000,
+                      help="how many tickets to test")
+    parser.add_option("--list", action="store_true", dest="list",
+                      default=False,
                       help="only write informations about tickets "
                            "that would be tested in the form: "
                            "[ticket id] [rating] [ticket title]")
-    parser.add_option("--skip-base", action="store_true", dest="skip_base",
-                      default=False,
-                      help="whether to check that the base is errorless")
+    parser.add_option("--ticket", dest="ticket",
+                      default=None,
+                      help="test only a list of tickets, for example"
+                           " '12345,19876'")
+
+    # options that are passed to the patchbot via the class "options"
+    # first two attribute options
     parser.add_option("--dry-run", action="store_true", dest="dry_run",
                       default=False)
     parser.add_option("--plugin-only", action="store_true", dest="plugin_only",
                       default=False,
                       help="run the patchbot in plugin-only mode")
-    parser.add_option("--cleanup", action="store_true", dest="cleanup",
-                      default=False,
-                      help="whether to cleanup the temporary files")
+    # then four other options that will be stored in conf
+    parser.add_option("--no-banner", action="store_true", dest="no_banner",
+                      help="wether to print the utf8 banner")
+    parser.add_option("--owner", dest="owner",
+                      help="name and email of the human behind the bot")
     parser.add_option("--safe-only", action="store_true", dest="safe_only",
-                      default=True,
                       help="whether to run the patchbot in safe-only mode")
+    parser.add_option("--skip-base", action="store_true", dest="skip_base",
+                      help="whether to check that the base is errorless")
 
     (options, args) = parser.parse_args(args)
 
+    # path to the json config file or None
     conf_path = options.config and os.path.abspath(options.config)
+
+    patchbot = Patchbot(os.path.abspath(options.sage_root),
+                        options.server,
+                        conf_path,
+                        options=options)
+    conf = patchbot.get_config()
+
     if options.ticket:
+        # only test the given list of tickets
         tickets = [int(t) for t in options.ticket.split(',')]
         count = len(tickets)
     else:
         tickets = None
         count = int(options.count)
 
-    patchbot = Patchbot(os.path.abspath(options.sage_root), options.server,
-                        conf_path, dry_run=options.dry_run,
-                        plugin_only=options.plugin_only, options=options)
-    conf = patchbot.get_config()
-
     if options.list:
         # the option "--list" allows to see tickets that will be tested
         patchbot.get_one_ticket(verbose=1)
@@ -1302,7 +1396,8 @@ def main(args):
     ensure_free_space(options.sage_root)
 
     if conf['use_ccache']:
-        do_or_die("'%s'/sage -i ccache" % options.sage_root, exn_class=ConfigException)
+        do_or_die("'%s'/sage -i ccache" %
+                  options.sage_root, exn_class=ConfigException)
         # If we rebuild the (same) compiler we still want to share the cache.
         os.environ['CCACHE_COMPILERCHECK'] = '%compiler% --version'
 
