@@ -1,3 +1,4 @@
+# global python imports
 import os
 import sys
 import bz2
@@ -21,20 +22,21 @@ try:
 except ImportError:
     from urllib.parse import quote
 
+# imports from patchbot sources
 from trac import scrape
-import patchbot
-import db
-
-from db import tickets
 from util import (now_str, current_reports, latest_version,
                   compare_version)
+from patchbot import filter_on_authors, prune_pending
+
+import db
+from db import tickets
 
 IMAGES_DIR = '/home/patchbot/sage-patchbot/src/images/'
 # oldest version of sage about which we still care
 OLDEST = '7.0'
 
 # machines that are banned from posting their reports
-BLACKLIST = ['hera-OptiPlex-7010', 'librae']
+BLACKLIST = ['hera-OptiPlex-7010']
 
 
 def timed_cached_function(refresh_rate=60):
@@ -224,7 +226,7 @@ def ticket_list():
     limit = int(request.args.get('limit', 1000))
     print(query)
 
-    all = patchbot.filter_on_authors(tickets.find(query).sort(order).limit(limit), authors)
+    all = filter_on_authors(tickets.find(query).sort(order).limit(limit), authors)
     if 'raw' in request.args:
         # raw json file for communication with patchbot clients
         def filter_reports(all):
@@ -299,7 +301,7 @@ def machines():
         authors = request.args.get('authors').split(':')
     else:
         authors = None
-    all = patchbot.filter_on_authors(tickets.find(query).limit(100), authors)
+    all = filter_on_authors(tickets.find(query).limit(100), authors)
     machines = {}
     for ticket in all:
         for report in ticket.get('reports', []):
@@ -357,7 +359,7 @@ def render_ticket(ticket):
     base_reports = base_reports_by_machine_and_base()
 
     old_reports = list(info['reports'])
-    patchbot.prune_pending(info)
+    prune_pending(info)
     if old_reports != info['reports']:
         db.save_ticket(info)
 
@@ -537,7 +539,7 @@ def render_ticket_status_svg(ticket):
         base = latest_version(info.get('reports', []))
 
     status = get_ticket_status(info, base=base)[1]  # single status
-    path = status_image_path(status, type='svg')
+    path = status_image_path(status, image_type='svg')
 
     # with no base
     response = make_response(open(path).read())
@@ -560,14 +562,14 @@ def post_report(ticket_id):
         report = json.loads(request.form.get('report'))
         assert (isinstance(report, dict)), "report is not a dict"
         for fld in ['status', 'spkgs', 'base', 'machine', 'time']:
-                assert (fld in report), "{} missing in report".format(fld)
+            assert (fld in report), "{} missing in report".format(fld)
 
         machine_name = report['machine'][-1]
         if machine_name in BLACKLIST:
             msg = 'machine {} is blacklisted'.format(machine_name)
             raise RuntimeError(msg)
 
-        patchbot.prune_pending(ticket, report['machine'])
+        prune_pending(ticket, report['machine'])
         ticket['reports'].append(report)
         db.logs.put(request.files.get('log'), _id=log_name(ticket_id, report))
         if 'retry' in ticket:
@@ -762,14 +764,14 @@ def status_image_svg(status):
     # stupid choice for the moment
     if len(liste) > 1:
         status = liste[0]
-    path = status_image_path(status, type='svg')
+    path = status_image_path(status, image_type='svg')
     response = make_response(open(path).read())
     response.headers['Content-type'] = 'image/svg+xml'
     response.headers['Cache-Control'] = 'max-age=3600'
     return response
 
 
-def status_image_path(status, type='png'):
+def status_image_path(status, image_type='png'):
     """
     Return the blob image address for a single status
 
@@ -777,7 +779,7 @@ def status_image_path(status, type='png'):
 
     For example, the result for 'TestsPassed' should be images/green-blob.png
     """
-    if type == 'png':
+    if image_type == 'png':
         return IMAGES_DIR + '{}-blob.png'.format(status_colors[status])
     else:
         return IMAGES_DIR + 'icon-{}.svg'.format(status)
